﻿using System.IdentityModel.Tokens.Jwt;
using System.Reflection;
using FluentValidation;
using FluentValidation.AspNetCore;
using Hotline.Api.Filters;
using Hotline.Application;
using Hotline.Application.Contracts;
using Hotline.CacheManager;
using Hotline.CallCenter.Devices;
using Hotline.NewRock;
using Hotline.Repository.SqlSugar;
using Hotline.Settings;
using Identity.Admin.HttpClient;
using Mapster;
using MapsterMapper;
using MediatR;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.IdentityModel.Tokens;
using Microsoft.OpenApi.Models;
using Serilog;
using XF.Domain.Dependency;

namespace Hotline.Api;

internal static class StartupExtensions
{
    const string CorsOrigins = "CorsOrigins";
    internal static WebApplication ConfigureServices(this WebApplicationBuilder builder)
    {
        var services = builder.Services;
        var configuration = builder.Configuration;

        services.AddHttpContextAccessor();

#if DEBUG
        builder.WebHost.UseUrls("http://192.168.100.62:50110", "http://localhost:50110");
#endif

        services.Configure<DeviceConfigs>(d => configuration.GetSection("DeviceConfigs").Bind(d));

        services.Configure<WorkTimeSettings>(d => configuration.GetSection("WorkTimeSettings").Bind(d));

        // Add services to the container.
        services
            .BatchInjectServices()
            .AddApplication()
            ;

        services.AddIdentityClient(
            new IdentityClientConfiguration("http://identity.fengwo.com", "http://open.identity.fengwo.com"),
            d =>
            {
                d.ClientId = "hotline_server";
                d.ClientSecret = "ce2fae0e-f0f6-46d6-bd79-1f1a31dff494";
                d.ClientScope = "identity.admin_api";
            });

        JwtSecurityTokenHandler.DefaultMapInboundClaims = false;
        services.AddAuthentication(JwtBearerDefaults.AuthenticationScheme)
            .AddJwtBearer(JwtBearerDefaults.AuthenticationScheme, d =>
            {
                d.Authority = "http://identity.fengwo.com"; //todo
                d.RequireHttpsMetadata = false;
                d.TokenValidationParameters = new TokenValidationParameters
                {
                    ValidateAudience = false
                };
            })
            ;

        services.AddControllers(options =>
        {
            //options.Filters.Add<TempTokenFilter>();
            options.Filters.Add<UnifyResponseFilter>();
            options.Filters.Add<UserFriendlyExceptionFilter>();
        });
        // Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
        services.AddEndpointsApiExplorer();
        services.AddSwaggerGen(c =>
        {
            //添加文档
            c.SwaggerDoc("v1", new OpenApiInfo() { Title = "Hotline Api", Version = "v1.0", Description = "城市热线api"});
            //使用反射获取xml文件，并构造出文件的路径
            var xmlFile = "document.xml";
            //var xmlFile = $"{Assembly.GetExecutingAssembly().GetName().Name}.xml";
            var xmlPath = Path.Combine(AppContext.BaseDirectory, xmlFile);
            // 启用xml注释. 该方法第二个参数启用控制器的注释，默认为false.
            c.IncludeXmlComments(xmlPath, true);

            var scheme = new OpenApiSecurityScheme()
            {
                Description = "Authorization header. \r\nExample: 'Bearer ***'",
                Reference = new OpenApiReference
                {
                    Type = ReferenceType.SecurityScheme,
                    Id = "Authorization"
                },
                Scheme = "oauth2",
                Name = "Authorization",
                In = ParameterLocation.Header,
                Type = SecuritySchemeType.ApiKey,
            };
            c.AddSecurityDefinition("Authorization", scheme);
            var requirement = new OpenApiSecurityRequirement();
            requirement[scheme] = new List<string>();
            c.AddSecurityRequirement(requirement);
        });

        //signalR
        builder.Services.AddSignalR().AddStackExchangeRedis(configuration.GetConnectionString("Redis"), options =>
        {
            options.Configuration.ChannelPrefix = "callcenter:signalR:";
        });

        /* CORS */
        services.AddCors(options =>
        {
            options.AddPolicy(name: CorsOrigins,
                builder =>
                {
                    var origins = configuration.GetSection("Cors:Origins").Get<string[]>();
                    builder.SetIsOriginAllowed(a =>
                        {
                            return origins.Any(origin => origin.StartsWith("*.", StringComparison.Ordinal)
                                ? a.EndsWith(origin[1..], StringComparison.Ordinal)
                                : a.Equals(origin, StringComparison.Ordinal));
                        })
                        .AllowAnyHeader()
                        .AllowAnyMethod()
                        .AllowCredentials();
                });
        });

        //mapster
        var config = TypeAdapterConfig.GlobalSettings;
        services.AddSingleton(config);
        services.AddScoped<IMapper, ServiceMapper>();

        //mediatr
        services.AddMediatR(Assembly.GetExecutingAssembly(), typeof(ApplicationStartupExtensions).Assembly);

        //迅时IPPBX
        var deviceConfigs = configuration.GetSection("DeviceConfigs").Get<DeviceConfigs>();
        services.AddNewRock(deviceConfigs.Address);

        //sqlsugar
        services.AddSqlSugar(configuration, "CallCenter");

        //cache
        services.AddCache(d =>
            {
                d.ConnectionString = configuration.GetConnectionString("Redis");
                d.Prefix = "CallCenter";
            });

        //validator
        services.AddFluentValidationAutoValidation(config =>
        {
            config.DisableDataAnnotationsValidation = true;
        })
            .AddValidatorsFromAssembly(typeof(AppContractsStartupExtensions).Assembly);
        return builder.Build();
    }

    internal static WebApplication ConfigurePipelines(this WebApplication app)
    {
        app.UseSerilogRequestLogging();

        var swaggerEnable = app.Configuration.GetSection("Swagger").Get<bool>();
        // Configure the HTTP request pipeline.
        if (swaggerEnable)
        {
            app.UseSwagger();
            app.UseSwaggerUI();
        }


        app.UseCors(CorsOrigins);

        app.UseAuthentication();
        app.UseAuthorization();
        //app.MapHub<CallCenterHub>("/hubs/callcenter");
        //app.UseMiddleware<TempTokenMiddleware>();

        app.MapControllers()
            .RequireAuthorization();

        return app;
    }
}